@@ -45,6 +45,7 @@
 #include "cc-permission-infobar.h"
 #include "cc-util.h"
 
+#define JOB_DEFAULT_PRIORITY  50
 #define RENEW_INTERVAL        500
 #define SUBSCRIPTION_DURATION 600
 
@@ -232,17 +233,10 @@ cc_printers_panel_constructed (GObject *object)
 
   shell = cc_panel_get_shell (CC_PANEL (self));
 
-  widget = (GtkWidget*)
-    gtk_builder_get_object (self->builder, "top-right-buttons");
-  cc_shell_embed_widget_in_header (shell, widget, GTK_POS_RIGHT);
-
   widget = (GtkWidget*)
     gtk_builder_get_object (self->builder, "search-bar");
-  g_signal_connect_object (shell,
-                           "key-press-event",
-                           G_CALLBACK (gtk_search_bar_handle_event),
-                           widget,
-                           G_CONNECT_SWAPPED);
+  gtk_search_bar_set_key_capture_widget (GTK_SEARCH_BAR (widget),
+                                         GTK_WIDGET (shell));
 }
 
 static void
@@ -457,7 +451,7 @@ on_cups_notification (GDBusConnection *connection,
     {
       g_autoptr(PpJob) job = NULL;
 
-      job = pp_job_new (job_id, NULL, 0, NULL);
+      job = pp_job_new (job_id, NULL, 0, JOB_DEFAULT_PRIORITY, NULL);
       pp_job_get_attributes_async (job,
                                    requested_attrs,
                                    cc_panel_get_cancellable (CC_PANEL (self)),
@@ -769,13 +763,12 @@ set_current_page (GObject      *source_object,
   update_sensitivity (user_data);
 }
 
-static void
-destroy_nonexisting_entries (PpPrinterEntry *entry,
-                             gpointer        user_data)
+static gboolean
+remove_nonexisting_entry (CcPrintersPanel *self,
+                          PpPrinterEntry  *entry)
 {
-  CcPrintersPanel  *self = (CcPrintersPanel *) user_data;
-  gboolean          exists = FALSE;
-  gint              i;
+  gboolean exists = FALSE;
+  gint     i;
 
   for (i = 0; i < self->num_dests; i++)
     {
@@ -787,10 +780,9 @@ destroy_nonexisting_entries (PpPrinterEntry *entry,
     }
 
   if (!exists)
-    {
-      g_hash_table_remove (self->printer_entries, pp_printer_entry_get_name (entry));
-      gtk_widget_destroy (GTK_WIDGET (entry));
-    }
+    g_hash_table_remove (self->printer_entries, pp_printer_entry_get_name (entry));
+
+  return !exists;
 }
 
 static void
@@ -801,6 +793,7 @@ actualize_printers_list_cb (GObject      *source_object,
   CcPrintersPanel        *self = (CcPrintersPanel*) user_data;
   GtkWidget              *widget;
   PpCupsDests            *cups_dests;
+  GtkWidget              *child;
   gboolean                new_printer_available = FALSE;
   g_autoptr(GError)       error = NULL;
   gpointer                item;
@@ -830,7 +823,16 @@ actualize_printers_list_cb (GObject      *source_object,
     gtk_stack_set_visible_child_name (GTK_STACK (widget), "printers-list");
 
   widget = (GtkWidget*) gtk_builder_get_object (self->builder, "content");
-  gtk_container_foreach (GTK_CONTAINER (widget), (GtkCallback) destroy_nonexisting_entries, self);
+  child = gtk_widget_get_first_child (widget);
+  while (child)
+    {
+      GtkWidget *next = gtk_widget_get_next_sibling (child);
+
+      if (remove_nonexisting_entry (self, PP_PRINTER_ENTRY (child)))
+        gtk_list_box_remove (GTK_LIST_BOX (widget), child);
+
+      child = next;
+    }
 
   for (i = 0; i < self->num_dests; i++)
     {
@@ -926,7 +928,7 @@ printer_add_async_cb (GObject      *source_object,
                                                    _("Failed to add new printer."));
           g_signal_connect (message_dialog,
                             "response",
-                            G_CALLBACK (gtk_widget_destroy),
+                            G_CALLBACK (gtk_window_destroy),
                             NULL);
           gtk_widget_show (message_dialog);
         }
@@ -957,22 +959,22 @@ new_printer_dialog_response_cb (GtkDialog *_dialog,
                                 self);
     }
 
-  gtk_widget_destroy (GTK_WIDGET (pp_new_printer_dialog));
+  gtk_window_destroy (GTK_WINDOW (pp_new_printer_dialog));
   self->pp_new_printer_dialog = NULL;
 }
 
 static void
 printer_add_cb (CcPrintersPanel *self)
 {
-  GtkWidget *toplevel;
+  GtkNative *native;
 
-  toplevel = gtk_widget_get_toplevel (GTK_WIDGET (self));
+  native = gtk_widget_get_native (GTK_WIDGET (self));
   self->pp_new_printer_dialog = pp_new_printer_dialog_new (self->all_ppds_list,
                                                            new_printer_dialog_response_cb,
                                                            self);
 
   gtk_window_set_transient_for (GTK_WINDOW (self->pp_new_printer_dialog),
-                                            GTK_WINDOW (toplevel));
+                                            GTK_WINDOW (native));
 
   gtk_widget_show (GTK_WIDGET (self->pp_new_printer_dialog));
 }
@@ -1124,11 +1126,13 @@ filter_function (GtkListBoxRow *row,
   g_autofree gchar       *name = NULL;
   g_autofree gchar       *location = NULL;
   GList                  *iter;
+  const gchar            *search_text;
 
   search_entry = (GtkWidget*)
     gtk_builder_get_object (self->builder, "search-entry");
+  search_text = gtk_editable_get_text (GTK_EDITABLE (search_entry));
 
-  if (gtk_entry_get_text_length (GTK_ENTRY (search_entry)) == 0)
+  if (g_utf8_strlen (search_text, -1) == 0)
     {
       retval = TRUE;
     }
@@ -1137,7 +1141,7 @@ filter_function (GtkListBoxRow *row,
       name = cc_util_normalize_casefold_and_unaccent (pp_printer_entry_get_name (entry));
       location = cc_util_normalize_casefold_and_unaccent (pp_printer_entry_get_location (entry));
 
-      search = cc_util_normalize_casefold_and_unaccent (gtk_entry_get_text (GTK_ENTRY (search_entry)));
+      search = cc_util_normalize_casefold_and_unaccent (search_text);
 
       retval = strstr (name, search) != NULL;
       if (location != NULL)
@@ -1309,6 +1313,10 @@ Please check your installation");
                       self);
 
   pp_cups_connection_test_async (self->cups, cc_panel_get_cancellable (CC_PANEL (self)), connection_test_cb, self);
-  gtk_container_add (GTK_CONTAINER (self), top_widget);
-  gtk_widget_show (GTK_WIDGET (self));
+  cc_panel_set_content (CC_PANEL (self), top_widget);
+
+  widget = (GtkWidget*)
+    gtk_builder_get_object (self->builder, "top-right-buttons");
+  adw_header_bar_pack_end (ADW_HEADER_BAR (cc_panel_get_titlebar (CC_PANEL (self))),
+                           widget);
 }
